# include	<stdio.h>
# include	<stdlib.h>
# include	<math.h>

# ifdef MSDOS
# include	<pi.h>
# endif

# include	"degtrig.h"
# include	"tidevect.h"

/*
 * Program to work out error between calculated Admiralty values and those
 *	given in the tables.
 */

typedef struct {
	int	angle;
	double	mag;
} Component;

char *mnames[] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun",
		   "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };
int mlengths[] = { 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };

static double	showerr( Vector actual, Component expected );
static char	buffer[128];

int
main( int argc, char *argv[] )
{
	char		*cptr;
	int		d, m, y, N = 0;
	Factors		*fptr;
	Component	m2, s2, k1, o1;
	double		m2err = 0.0, s2err = 0.0, k1err = 0.0, o1err = 0.0;
	FILE		*in;

	y = 93;
	if ((y & 0x03) == 0) mlengths[1] = 29;

	if ((in = fopen("months.txt", "r")) == NULL) {
		fprintf(stderr, "Can't read 'months.txt'\n");
		return -1;
	}

	for (m = 1; m <= 12; m++) {
		printf("%s %d:\t    M2\t\t    S2\t\t    K1\t\t    O1\n", mnames[m-1], y);

		for (d = 1; d <= mlengths[m-1]; d++) {
			printf("%2d:\t", d);
			fptr = mkfactors(0.0, d, m, y);

			/* read actual values */
			while (1) {
				if ((cptr = fgets(buffer, sizeof buffer, in)) == NULL) {
					fprintf(stderr, "'months.txt' expired early!\n");
					goto breakout;
				}

				if (*cptr != '\n')
					break;
			}

			/* check day number */
			if (d != ( int ) strtol(cptr, &cptr, 10)) {
				fprintf(stderr, "Surprise day value!\n");
				return -1;
			}

			/* decode actual values */
			m2.angle = ( int ) strtol(cptr, &cptr, 10);
			m2.mag = strtod(cptr, &cptr);
			s2.angle = ( int ) strtol(cptr, &cptr, 10);
			s2.mag = strtod(cptr, &cptr);
			k1.angle = ( int ) strtol(cptr, &cptr, 10);
			k1.mag = strtod(cptr, &cptr);
			o1.angle = ( int ) strtol(cptr, &cptr, 10);
			o1.mag = strtod(cptr, &cptr);

			/* print error */
			m2err += showerr(fptr->m2, m2);
			putchar('\t');
			s2err += showerr(fptr->s2, s2);
			putchar('\t');
			k1err += showerr(fptr->k1, k1);
			putchar('\t');
			o1err += showerr(fptr->o1, o1);
			putchar('\n');
			N++;

			/* skip a line every 4 */
			if ((d & 0x03) == 0)
				printf("\n");
		}

		printf("\n\n");
	}

breakout:
	m2err /= ( double ) N;
	s2err /= ( double ) N;
	k1err /= ( double ) N;
	o1err /= ( double ) N;

	printf("Errors:\t%8.4f\t%8.4f\t%8.4f\t%8.4f\n", m2err, s2err, k1err, o1err);
	return 0;
}

static double
showerr( Vector actual, Component expected)
{
	double	D, d, Dsin_d, Dcos_d;

	/* now work out difference vector */
	/* tables count the other way, so correct that */
	if ((d = ( double ) expected.angle) != 0.0)
		d = 360.0 - d;

	D = expected.mag;

	Dsin_d = D*sine(d) - actual.t_sine;
	Dcos_d = D*cosine(d) - actual.t_cosine;

	/* finally, difference vector itself.. */
	d = arctan2(Dsin_d, Dcos_d);
	D = sqrt(Dsin_d*Dsin_d + Dcos_d*Dcos_d);

	printf("%03d %5.3f", ( int ) d, D);
	return D;
}
