# include	<stdio.h>
# include	<math.h>

# include	"degtrig.h"
# include	"tidevect.h"

/* number of days from Jan 1st to start of a given month, std year */
int	mtot[] = {0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334};

# ifdef MSDOS
# include	<pi.h>
# endif

int		setvect( Vector *, double, double );
int 		addvect( Vector *, double, double );
static double	limit( double n );
static int	ilimit( double n );

/*
 * Fhddr - if debugging, print a title line.
 */
void
fhddr( void )
{

# ifdef ASTRO
	printf("  MM  Q1  O1  M1  P1  K1  J1 2n2  v2  u2  N2  M2  l2  L2  K2  T2\n");
# endif
	return;
}

/*
 * Mkfactors - finds the Admiralty tidal factors for 00:00 on a given day.
 *	Caution: returns pointer to static.
 */
Factors *
mkfactors( double time, int d, int m, int y )
{
	int	tmp;
	double	elapsed, e_rot;
	double	s, h, p, N, pdash, x, C;
	double	fSin_u, fCos_u, f, u;
	double	fQ1, fM1, fK1, fJ1, fM2, fL2, fK2;
# ifdef ASTRO
	double	fMM;
# endif
	static Factors	ret;

	/* correct d to be day since epoch */
	d += (y-1980)*365;	/* 'normal' days since epoch */
	d += mtot[m-1];		/* 'normal' days this year so far */

	/* tally leap days since Jan 0.0 1980 */
	do {	
		if (m > 2) {
			/* march - december: is this a leap year? */
			if (y%100 == 0) {
				/* centennial year */
				if (((y/100) & 0x03) == 0)
					d++;		/* add leap day */
			}
			else {
				/* 'normal year' */
				if ((y & 0x03) == 0)
					d++;		/* add leap day */
			}
		}
		
		m = 13;	/* force leap-day calculation on previous years */
		y--;	/* go to last year */
	} while (y >= 1980);
	
	elapsed = d + time/24.0;
	e_rot = 15.0*time;

	/* epoch 1980.0 */
	/* h = sun's mean longitude */
	/* s = moon's mean longitude */
	/* p = longitude of moon's perigee */
	/* N = longitude of moon's ascending node */
	h = limit(278.833540 + 0.985647332*elapsed);	/* year=365.2422days */
	s = limit( 64.975464 + 13.1763966*elapsed);
	p = limit(349.383063 + 0.11140410*elapsed);
	N = limit(151.950429 - 0.05295390*elapsed);

	/* true longitude of moon */
	x = s + 0.110*sine(s - p) + 0.022*sine(s - 2*h + p)
	      + 0.011*sine(2*s - 2*h) + 0.004*sine(2*s - 2*p);

	/* c/r (parallax) of moon - cubed */
	C = 1.0 + 0.055*cosine(s - p) + 0.01*cosine(s - 2*h + p)
		+ 0.008*cosine(2*s - 2*h) + 0.003*cosine(2*s - 2*p);
	C *= C * C;

# ifdef DEBUG
	printf("h=%.4f, s=%.4f, p=%.4f, N=%.4f, x=%.4f, C=%.4f\n", h, s, p, N, x, C);
# endif

	/* moon phase */
	ret.mphase = limit(h - x);

	/* valid for 20th century.. */
	/* pdash = ecliptic longitude of sun at perigee */
	pdash = 282.596403;

# ifdef ASTRO
	/* Slow tides.. */
	/* MM */
	fMM = 1.000 - 0.130*cosine(N);
	u = 0.0;
	printf("%4d", ilimit(0.544375*time + s - p + u));
# endif

	/* Daily tides.. */
	/* Q1 */
	fQ1 = 1.009 + 0.187*cosine(N) - 0.015*cosine(2.0*N);
	u = 10.8*sine(N) - 1.3*sine(2.0*N) + 0.2*sine(3*N);
/*	e_rot = 13.398661*time;	*/
	tmp = setvect(&ret.o1, limit(e_rot + h - 3.0*s + p - 90.0 + u), fQ1*0.19098);
# ifdef ASTRO
	printf("%4d", tmp);
# endif

	/* O1 */
/*	e_rot = 13.943036*time;	*/
# ifdef MODIFY_O1
	/* O1 modified for true lunar longitude & parallax */
	addvect(&ret.o1, limit(e_rot + h - 2.0*x - 90.0 + u), fQ1*C);
	tmp = ilimit(e_rot + h - 2.0*s - 90.0 + u);
# else
	tmp = addvect(&ret.o1, limit(e_rot + h - 2.0*s - 90.0 + u), fQ1);
# endif
# ifdef ASTRO
	printf("%4d", tmp);
# endif

	/* M1 */
/*	e_rot = 14.496694*time;	*/
	fCos_u = 2.0*cosine(p) + 0.4*cosine(p - N);
	fSin_u = sine(p) + 0.2*sine(p - N);
	fM1 = sqrt(fSin_u*fSin_u + fCos_u*fCos_u);
	u = arctan2(fSin_u, fCos_u);
/*	e_rot = 14.496694*time;	*/
	tmp = addvect(&ret.o1, limit(e_rot + h - s + 90.0 + u), 0.0788*fM1/fQ1);
# ifdef ASTRO
	printf("%4d", tmp);
# endif

# ifdef UNUSED
	/* K1 (lunar only) */
/*	e_rot = 15.041069*time;	*/
	addvect(&ret.o1, limit(e_rot + h + 90.0 + u), 0.4458*fM1/fQ1);
# endif

	/* P1 */
	f = 1.0;
	u = 0.0;
/*	e_rot = 14.958931*time;	*/
	tmp = setvect(&ret.k1, limit(e_rot - h - 90.0 + u), f*0.33145);
# ifdef ASTRO
	printf("%4d", tmp);
# endif

	/* K1 (lunar & solar) */
	fK1 = 1.006 + 0.115*cosine(N) - 0.009*cosine(2.0*N);
	u = -8.9*sine(N) + 0.7*sine(2.0*N);
/*	e_rot = 15.041069*time;	*/
	tmp = addvect(&ret.k1, limit(e_rot + h + 90.0 + u), fK1);
# ifdef ASTRO
	printf("%4d", tmp);
# endif

	/* J1 */
	fJ1 = 1.013 + 0.168*cosine(N) - 0.017*cosine(2.0*N);
	u = -12.9*sine(N) + 1.3*sine(2.0*N) - 0.2*sine(3.0*N);
/*	e_rot = 15.585443*time;	*/
	tmp = addvect(&ret.k1, limit(e_rot + h + s - p + 90.0 + u), fJ1*0.056497);
# ifdef ASTRO
	printf("%4d", tmp);
# endif

	/* Diurnal tides .... */
	/* these move twice as fast */
	e_rot *= 2;

	/* 2n2 */
	fM2 = 1.0 - 0.037*cosine(N);
	u = -2.1*sine(N);
/*	e_rot = 27.895355*time;	*/
	tmp = setvect(&ret.m2, limit(e_rot + 2.0*h - 4.0*s +2.0*p + u), fM2*0.0258668);
# ifdef ASTRO
	printf("%4d", tmp);
# endif

	/* v2 */
/*	e_rot = 28.512583*time;	*/
	tmp = addvect(&ret.m2, limit(e_rot + 4.0*h - 3.0*s - p + u), fM2*0.0375343);
# ifdef ASTRO
	printf("%4d", tmp);
# endif

	/* u2 */
/*	e_rot = 27.968208*time;	*/
	tmp = addvect(&ret.m2, limit(e_rot + 4.0*(h - s) + u), fM2*0.0241056);
# ifdef ASTRO
	printf("%4d", tmp);
# endif

	/* N2 */
/*	e_rot = 28.439730*time;	*/
	tmp = addvect(&ret.m2, limit(e_rot + 2.0*h - 3.0*s + p + u), fM2*0.1936158);
# ifdef ASTRO
	printf("%4d", tmp);
# endif

	/* M2 */
/*	e_rot = 28.984104*time;	*/
	tmp = addvect(&ret.m2, limit(e_rot + 2.0*(h - s) + u), fM2);
# ifdef ASTRO
	printf("%4d", tmp);
# endif

	/* lambda2 */
/*	e_rot = 29.455625*time;	*/
	tmp = addvect(&ret.m2, limit(e_rot + p - s + 180.0 + u), fM2*0.00726472);
# ifdef ASTRO
	printf("%4d", tmp);
# endif

	/* L2 */
	fCos_u = 1.0 - 0.25*cosine(2.0*p) - 0.11*cosine(2.0*p - N) - 0.02*cosine(2.0*(p - N)) - 0.04*cosine(N);
	fSin_u = -0.25*sine(2.0*p) - 0.11*sine(2.0*p - N) - 0.02*sine(2.0*(p - N)) - 0.04*sine(N);
	fL2 = sqrt(fSin_u*fSin_u + fCos_u*fCos_u);
	u = arctan2(fSin_u, fCos_u);
/*	e_rot = 29.528479*time;	*/
	tmp = addvect(&ret.m2, limit(e_rot + 2.0*h - s - p + 180.0 + u), fL2*0.0276279);
# ifdef ASTRO
	printf("%4d", tmp);
# endif

	/* S2 */
	f = 1.0;
	u = 0.0;
/*	e_rot = 30.000000*time;	*/
	setvect(&ret.s2, limit(e_rot + u), f);

	/* K2 */
# define	OOPS
# ifdef OOPS
	/* these factors are supposedly for the *lunar* K2 */
	fK2 = 1.024 + 0.286*cosine(N) + 0.008*cosine(2.0*N);
	u = -17.7*sine(N) + 0.7*sine(2.0*N);
# else
	/* the *solar* component of K2 should merge into S2, lunar into M2 */
	fK2 = 1.0;
	u = 0.0;
# endif
/*	e_rot = 30.082137*time;	*/
	tmp = addvect(&ret.s2, limit(e_rot + 2.0*h + u), fK2*0.2718676);
# ifdef ASTRO
	printf("%4d", tmp);
# endif

	/* T2 */
/*	e_rot = 29.958933*time;	*/
	tmp = addvect(&ret.s2, limit(e_rot + pdash - h + u), fK2*0.0591);
# ifdef ASTRO
	printf("%4d\n", tmp);
# endif
# ifdef DEBUG
	printf("%.2f %.2f %.2f %.2f %.2f %.2f %.2f %.2f\n", fMM, fQ1, fM1, fK1, fJ1, fM2, fL2, fK2);
# endif
	return &ret;
}

static double
limit( double n )
{

	while (n < 0.0)
		n += 360.0;

	while (n >= 360.0)
		n -= 360.0;

	return n;
}

static int
ilimit( double n )
{

	return ( int ) ( limit(n) + 0.5 );
}

int
setvect( Vector *v, double angle, double factor )
{

	v->t_sine = sine(angle) * factor;
	v->t_cosine = cosine(angle) * factor;
	return ( int ) ( angle + 0.5 );
}

int
addvect( Vector *v, double angle, double factor )
{

	v->t_sine += sine(angle) * factor;
	v->t_cosine += cosine(angle) * factor;
	return ( int ) ( angle + 0.5 );
}


void
pr_mphase( double f, int w )
{
	char	*messg;

	/* Examine moon phase +/- 'w' degrees */
	if (f > (360.0-w) || f < ( double )w)
		messg = "New moon";
	else if (f > (270.0+w))
		messg = "Waxing Crescent";
	else if (f > (270.0-w))
		messg = "First Quarter";
	else if (f > (180.0+w))
		messg = "Waxing Gibbous";
	else if (f > (180.0-w))
		messg = "Full moon";
	else if (f > (90.0+w))
		messg = "Waning Gibbous";
	else if (f > (90.0-w))
		messg = "Last Quarter";
	else
		messg = "Waning Crescent";

	if (w < 5)
		/* accuracy required */
		printf("%s (%.1f%%)\n", messg, 100.0*0.5*(1.0 - cosine(f)));
	else
		printf("%s\n", messg);

	return;
}

/*
 * Pr_tide - tide time is given in GMT, 'tz' is what to subtract to get
 * local time.
 */
void
pr_tide( Tide tide, int tz )
{
	int	hrs, mins;

	if (tide.height == 0.0) {
		printf("Invalid tide\n");
		return;
	}

/*	printf("[%d]", tz); */
	tide.time -= ( double ) tz;
	hrs = ( int ) tide.time;
	mins = ( int )((tide.time - ( double )hrs) * 60.0);

	/* hour may roll-over due to timezone correction */
	if (hrs >= 24)
		hrs -= 24;
	else if (hrs < 0)
		hrs += 24;

	printf("%02d:%02d  %4.1f mtrs\n", hrs, mins, tide.height);
	return;
}
