# include	<math.h>
# include	"degtrig.h"
# include	"tidevect.h"

#ifdef MSDOS
# include	<pi.h>

Vector	vshift( Vector, int );
Vector	vscale( Vector, double );
#else
Vector	vshift();
Vector	vscale();
#endif

Tide
mktide(time, day, month, year, port)
double time;
Port *port;
{
	double		ml;
	Vector		m2, s2, k1, o1;
	double		R_sin_r, R_cos_r;
	double		D4_cos_d4, D6_cos_d6;
	double		D4_sin_d4, D6_sin_d6;
	double		r, d4, d6;
	double		R, D4, D6;
	Factors		*fptr;
	Tide		tide;

	/* get equilibrium tidal factors for given time */
	fptr = mkfactors(time, day, month, year);

	/* hot-wire results! */
	fptr->m2.t_sine = -(fptr->m2.t_sine);
	fptr->s2.t_sine = -(fptr->s2.t_sine);
	fptr->k1.t_sine = -(fptr->k1.t_sine);
	fptr->o1.t_sine = -(fptr->o1.t_sine);

	ml = port->z0;			/* line 3 */

	/* lines 6 thru 30 (ish) replaced by vector arithmetic */
	/* factors are correct for this time & date */
	/* shift them from equilibrium conditions */
	m2 = vshift(fptr->m2, port->m2g);
	s2 = vshift(fptr->s2, port->s2g);
	k1 = vshift(fptr->k1, port->k1g);
	o1 = vshift(fptr->o1, port->o1g);

	/* adjust magnitudes away from equilibrium conditions */
	m2 = vscale(m2, port->m2H);
	s2 = vscale(s2, port->s2H);
	k1 = vscale(k1, port->k1H);
	o1 = vscale(o1, port->o1H);

	/* combine diurnal tides to get at some fast-moving tides */
	R_sin_r = m2.t_sine + s2.t_sine;
	R_cos_r = m2.t_cosine + s2.t_cosine;	/* line 32 */
# ifdef DEBUG
	printf("%6.4f, %6.4f (line 32)\n", R_sin_r, R_cos_r);
# endif

	r = arctan2(R_sin_r, R_cos_r);
	R = sqrt(R_sin_r*R_sin_r + R_cos_r*R_cos_r);	/* line 33 */
# ifdef DEBUG
	printf("%6.2f, %6.4f (line 33)\n", r, R);
# endif

	d4 = r*2.0 + port->f4;
	D4 = R*R * port->F4;
	D4_cos_d4 = D4 * cosine(d4);	/* line 36 */
	D4_sin_d4 = 2.0 * D4 * sine(d4); /* d/dr of the above */
# ifdef DEBUG
	printf("%6.2f, %6.4f, %6.4f (line 36)\n", d4, D4, D4_cos_d4);
# endif

	d6 = r*3.0 + port->f6;
	D6 = R*R*R * port->F6;
	D6_cos_d6 = D6 * cosine(d6);	/* line 39 */
	D6_sin_d6 = 3.0 * D6 * sine(d6); /* d/dr of the above */
# ifdef DEBUG
	printf("%6.2f, %6.4f, %6.4f (line 39)\n", d6, D6, D6_cos_d6);
# endif

	/* sum of cosines is tide height, sum of sines is slope (d/dr)*/
	tide.height = ml + k1.t_cosine + o1.t_cosine + R_cos_r + D4_cos_d4 + D6_cos_d6;
	tide.slope = k1.t_sine + o1.t_sine + R_sin_r + D4_sin_d4 + D6_sin_d6;
	tide.time = time;

# ifdef CIVIL
	/* finally, civil tables use funny datum */ 
	/* this seems to fix it.. */
	tide.height += 3.06;
# endif
	return tide;
}

Vector
vshift(v, angle)
Vector v;
int angle;
{
	double	asin, acos;
	Vector	ret;

	asin = sine(( double ) angle);
	acos = cosine(( double ) angle);

	ret.t_sine = v.t_sine*acos + v.t_cosine*asin;
	ret.t_cosine = v.t_cosine*acos - v.t_sine*asin;
	return ret;
}

Vector
vscale(v, factor)
Vector v;
double factor;
{

	v.t_sine *= factor;
	v.t_cosine *= factor;
	return v;
}
