# include	<math.h>
# include	"degtrig.h"

# define	ESUN		0.01672

double limit(double);

/*
 * Moonlong - calculate true longitude of moon from:
 *
 * s =		mean longitude of moon.
 * p =		mean longitude of moon's perigee.
 * h =		mean longitude of sun.
 * pdash =	mean longitude of sun's perigee.
 */
double
moonlong(s, p, h, pdash)
double s, p, h, pdash;
{
	double	M, Lambdasun, MM, Ev, Ae, A3, MmP, mEc, A4, lP, V;

# ifdef DEBUG
	printf("%6.2f, %6.2f, %6.2f, %6.2f", s, p, h, pdash);
# endif

	M = limit(h - pdash);				/* mean anomaly (sun) */
	Lambdasun = limit(h + (360.0*ESUN/PI)*sine(M));	/* true anomaly (sun) */
	MM = limit(s - p);				/* mean anomaly(moon) */
	Ev = 1.2739*sine(2.0*(s - Lambdasun) - MM);	/* evection */
	Ae = 0.1858*sine(M);				/* annual eqn */
	A3 = 0.37*sine(M);				/* correction */
	MmP = MM + Ev - Ae - A3;			/* correct anomaly */
	mEc = 6.2886*sine(MmP);				/* eqn of the centre */
	A4 = 0.214*sine(2.0*MmP);			/* another correction */
	lP = s + Ev + mEc - Ae + A4;			/* correct longitude */
	V = 0.6583*sine(2.0*(lP - Lambdasun));		/* variation */

# ifdef DEBUG
	printf(" = %6.2f, Lambdasun = %6.2f\n", lP + V, Lambdasun);
# endif

	return lP + V;					/* True Longitude! */
}
