# include	<stdio.h>
# include	<stdlib.h>
# include	<math.h>

# include	"tidevect.h"

static int	zeroslope( Tide tide );
static int	diffslope( Tide tide1, Tide tide2 );


Tideset
search4( int tz, int d, int m, int y, Port *portptr)
{
	int		ntides;
	double		time;
	Tide		prevtide, tide, *tptr;
	Tideset		retval;

	ntides = 0;
	tptr = &retval.t[0];

	/* in case there's no 2nd, 3rd or 4th tide */
	retval.t[1].height = 0.0;
	retval.t[2].height = 0.0;
	retval.t[3].height = 0.0;

	prevtide = mktide(time = 0.0 + tz, d, m, y, portptr);

	/* search between previous tide and 6hrs later */
	while (ntides < 4 && time < 24.0+tz) {
		/* step onwards 6 hours */
		time += 6.0;

		while (time <= 24.0+tz) {
			tide = mktide(time, d, m, y, portptr);

			if (zeroslope(prevtide)) {
				*tptr++ = prevtide;
				ntides++;
				break;
			}
			else if (zeroslope(tide)) {
				*tptr++ = tide;
				ntides++;
				break;
			}
			else if (diffslope(prevtide, tide)) {
				*tptr++ = search(prevtide, tide, d, m, y, portptr);
				ntides++;
				break;
			}
			else	
				/* widen window by an hour */
				time += 1.0;
		}

		prevtide = tide;
	}

	return retval;
}


Tide
search( Tide tide1, Tide tide2, int d, int m, int y, Port *portptr)
{
	double	dtime, dslope;
	Tide	tmptide;

	tmptide.height = 0.0;	/* flags illegal tide */

	do {
		dtime = tide2.time - tide1.time;
		dslope = fabs(tide1.slope - tide2.slope);
		dtime = tide1.time + (dtime/dslope)*fabs(tide1.slope);
		tmptide = mktide(dtime, d, m, y, portptr);

		if (diffslope(tide1, tmptide))
			tide2 = tmptide;
		else if (diffslope(tmptide, tide2))
			tide1 = tmptide;
		else {
			printf("Stuck!\n");
			return tmptide;
		}
	} while (!zeroslope(tmptide));

	return tmptide;
}

static int
zeroslope( Tide tide )
{

	/* don't need super accuracy! */
	return tide.slope < 0.001 && tide.slope > -0.001;
}

static int
diffslope( Tide tide1, Tide tide2 )
{
	register double	s1, s2;

	s1 = tide1.slope;
	s2 = tide2.slope;

	return (s1 < 0.0 && s2 > 0.0) || (s1 > 0.0 && s2 < 0.0);
}
