# include	<stdio.h>
# include	<math.h>
# include	<stdlib.h>
# include	<time.h>
# include	<string.h>
# include	<ctype.h>
# ifdef MSDOS
# include	<conio.h>
# endif

# include	"degtrig.h"
# include	"tidevect.h"
# include	"ports.h"

static void	usage( void );
static void	credits( void );
static char	*mnames[] = {	"Jan", "Feb", "Mar", "Apr", "May", "Jun",
				"Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };

int
main( int argc, char *argv[] )
{
# ifdef MSDOS
	char		tzbuffer[20];
# endif
	char		*tzptr, **ap = &argv[1];
	char		*portname = "Swansea King's Dock";
	int		tz, winterzone;
	time_t		now;
	Tide		tide, lasttide, nexttide;
	Port		*portptr = &swansea;
	Factors		*tmp;
	double		hrs;
	struct tm	*tptr;

	/* play safe */
	tzset();

	/* time returns GMT */
	now = time(( time_t * ) 0);
	tptr = localtime(&now);
	
	/* find 'winterzone', removing today's possible summer-time effects */	
	winterzone = ( int )(timezone / 3600L) + ((tptr->tm_isdst > 0)? 1: 0);
	
# ifdef MSDOS
	/* handle the TZ environment variable if possible */
	if ((tzptr = getenv("TZ")) && strlen(tzptr) >= 4) {
		/* valid-looking TZ exists */
		strncpy(tzbuffer, tzptr, sizeof tzbuffer);
		tz = atoi(tzptr+3);

		/* Isolate correct timezone mnemonic */
		if (tptr->tm_isdst) {
			/* find DST mnemonic - assume DST is -1 hour */
			tzptr += 3;
			if (*tzptr == '+' || *tzptr == '-')
				tzptr++;

			while (isdigit(*tzptr))
				tzptr++;

			tz--;
		}

		if (*tzptr == '\0') {
			/* No DST mnemonic given? Use normal one minus 1 */
			tzptr = tzbuffer;
			strcpy(tzbuffer+3, "-01:00");
		}
		else if (strlen(tzptr) > 3)
			/* null-terminate mnemonic */
			tzptr[3] = '\0';
	}
	else {
		/* must assume GMT */
		tzptr = "GMT";
		tz = 0;
	}
# else
	tzptr = tzname[(tptr->tm_isdst > 0)? 1: 0];
	tz = winterzone - ((tptr->tm_isdst > 0)? 1: 0);	
# endif

	/* parse option flags */
	while (argc > 1 && (*ap) && (*ap)[0] == '-') {
		switch ((*ap)[1]) {
		case 'B':
		case 'b':
			portptr = &burryport;
			portname = "Burry Port";
			break;

		case 'C':
		case 'c':
			portptr = &caswell;
			portname = "Caswell Bay";
			break;

		case 'H':
		case 'h':
		case '?':
			credits();
# ifdef MSDOS
			printf("More...");
			while (!kbhit());
			putchar('\r');
# endif
			usage();
			return -1;

		case 'M':
		case 'm':
			portptr = &mumbles;
			portname = "Mumbles Slip";
			break;

		case 'P':
		case 'p':
			portptr = &porttalbot;
			portname = "Port Talbot";
			break;

		case 'S':
		case 's':
			/* default port anyway */
			break;

		default:
			fprintf(stderr, "Unknown arg: %s\n", *ap);
			usage();
			return -1;
		}

		ap++;
		argc--;
	}

	if (argc != 1) {
		usage();
		return -1;
	}

	/* use GMT for tide calculation.. */
	hrs = tptr->tm_hour + (tptr->tm_min + tptr->tm_sec/60.0)/60.0 + tz;
	tide = mktide(hrs, tptr->tm_mday, tptr->tm_mon+1, tptr->tm_year+1900, portptr);

	/* use local time for printout */
	tptr = localtime(&now);
	printf("Time now: %02d:%02d %3s %02d %3s %04d\n", tptr->tm_hour, tptr->tm_min, tzptr, tptr->tm_mday, mnames[tptr->tm_mon], tptr->tm_year+1900);

	tmp = mkfactors(hrs, tptr->tm_mday, tptr->tm_mon+1, tptr->tm_year+1900);
	printf("Moon phase: ");
	pr_mphase(tmp->mphase, 2);

	printf("\nLocation: %s\n", portname);
	printf("Tide height = %4.1f mtrs ", tide.height);

	if (tide.slope < -0.05) {
		printf("and ebbing\n");
		lasttide = mktide(hrs-7.0, tptr->tm_mday, tptr->tm_mon+1, tptr->tm_year+1900, portptr);
		
		if (lasttide.slope < 0.0)
			/* oops - overshot! */
			lasttide = mktide(hrs-3.5, tptr->tm_mday, tptr->tm_mon+1, tptr->tm_year+1900, portptr);
			
		lasttide = search(lasttide, tide, tptr->tm_mday, tptr->tm_mon+1, tptr->tm_year+1900, portptr);
		printf("Last high tide: ");
		pr_tide(lasttide, tz);
		nexttide = mktide(hrs+7.0, tptr->tm_mday, tptr->tm_mon+1, tptr->tm_year+1900, portptr);
		
		if (nexttide.slope < 0.0)
			/* oops - overshot! */
			nexttide = mktide(hrs+3.5, tptr->tm_mday, tptr->tm_mon+1, tptr->tm_year+1900, portptr);
			
		nexttide = search(tide, nexttide, tptr->tm_mday, tptr->tm_mon+1, tptr->tm_year+1900, portptr);
		printf("Next low tide: ");
		pr_tide(nexttide, tz);
	}
	else if (tide.slope > 0.05) {
		printf("and rising\n");
		lasttide = mktide(hrs-7.0, tptr->tm_mday, tptr->tm_mon+1, tptr->tm_year+1900, portptr);
		
		if (lasttide.slope > 0.0)
			/* oops - overshot! */
			lasttide = mktide(hrs-3.5, tptr->tm_mday, tptr->tm_mon+1, tptr->tm_year+1900, portptr);
			
		lasttide = search(lasttide, tide, tptr->tm_mday, tptr->tm_mon+1, tptr->tm_year+1900, portptr);
		printf("Last low tide: ");
		pr_tide(lasttide, tz);
		nexttide = mktide(hrs+7.0, tptr->tm_mday, tptr->tm_mon+1, tptr->tm_year+1900, portptr);
		
		if (nexttide.slope > 0.0)
			/* oops - overshot! */
			nexttide = mktide(hrs+3.5, tptr->tm_mday, tptr->tm_mon+1, tptr->tm_year+1900, portptr);
			
		nexttide = search(tide, nexttide, tptr->tm_mday, tptr->tm_mon+1, tptr->tm_year+1900, portptr);
		printf("Next high tide: ");
		pr_tide(nexttide, tz);
	}
	else if (tide.height > portptr->z0)
		printf("(high tide)\n");
	else
		printf("(low tide)\n");
		
	return 0;
}

static void
credits( void )
{

	printf(
	"Swansea Bay & Gower Tide State Program V3.0, May 1996.\n"
	"Written by Steven Hosgood: Copyright (c) S.Hosgood, 1988,1994,1996.\n"
	"Data digitisation by Andrew Jenkins.\n"
	"\nBased upon:\n"
	"\tBritish Admiralty Simplified Harmonic Method, (document NP159)\n"
	"\nReferences:\n"
	"\tDoodson A.T & Warburg H.D: 'Admiralty Manual of Tides', [GC301.Adm]\n"
	"\tGodin G: 'The Analysis of Tides', [GC301.God]\n"
		);
	printf(
	"\nVersion History:\n"
	"1.0:\tOriginal, June 1988\n"
	"2.0:\t'M1' component included in combined 'O1' tide factor,\n"
	"\tAdded print-out of moon phase and percentage-illuminated,\n"
	"\tNew Swansea constants from Admiralty Tide Tables,\n"
	"\tAdded Mumbles, Burry Port, Port Talbot options, June 1994\n"
	"3.0:\tLinux port,\n"
	"\tAdded 'last tide' print-out,\n"
	"\tImproved tide search algorithm, May 1996\n"
	"4.0:\tFixed Linux/glibc DST-handling bugs,\n"
	"\tAdded experimental Caswell Bay coefficients, July 1999\n"
		);
	printf(
	"\nThis version of this program may freely be copied and distributed\n"
	"as long as the credits and copyright message are kept intact, and\n"
	"that no charge is made for the copies. No guarantees as to the\n"
	"accuracy of the program are made, and none are to be implied.\n"
	"The program is for entertainment purposes only.\n"
	"\n"
		);

	return;
}


static void
usage ( void )
{

	printf("Usage: tidenow [args]\n");
	printf("\nArgs:\n");
	printf("\t-? or -h\tHelp Screen\n");
	printf("\t-b\t\tBurry Port\n");
	printf("\t-m\t\tMumbles Slip\n");
	printf("\t-p\t\tPort Talbot\n");
	printf("\t-s\t\tSwansea Docks (default)\n");
	return;
}
