# include	<stdio.h>
# include	<math.h>
# include	<stdlib.h>
# include	<time.h>
# include	<string.h>
# include	<ctype.h>

# ifdef MSDOS
# include	<conio.h>
# endif

# include	"degtrig.h"
# include	"tidevect.h"
# include	"ports.h"

static void	usage( void );
static void	credits( void );

static char	*wdaynames[] = { "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat" };
static char	*mnames[] = {	"Jan", "Feb", "Mar", "Apr", "May", "Jun",
				"Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };

int
main( int argc, char *argv[] )
{
# ifdef MSDOS
	char		tzbuffer[20];
# endif
	char		*tzptr, **ap = &argv[1];
	int		tz, winterzone, d, m, y;
	time_t		now;
	struct tm	*tptr;
	Factors		*tmp;
	Port		*portptr = &swansea;
	char		*portname = "Swansea King's Dock";
	Tideset		tides;

	/* play safe */
	tzset();

	/* 'time' returns GMT, tptr interprets to civil time */
	now = time(( time_t * ) 0);
	tptr = localtime(&now);

	/* find 'winterzone', removing today's possible summer-time effects */	
	winterzone = ( int )(timezone / 3600L) + ((tptr->tm_isdst > 0)? 1: 0);
	
	/* parse option flags */
	while (argc > 1 && (*ap) && (*ap)[0] == '-') {
		switch ((*ap)[1]) {
		case 'B':
		case 'b':
			portptr = &burryport;
			portname = "Burry Port";
			break;

		case 'C':
		case 'c':
			portptr = &caswell;
			portname = "Caswell Bay";
			break;

		case 'H':
		case 'h':
		case '?':
			credits();
# ifdef MSDOS
			printf("More...");
			while (!kbhit());
			putchar('\r');
# endif
			usage();
			return -1;

		case 'M':
		case 'm':
			portptr = &mumbles;
			portname = "Mumbles Slip";
			break;

		case 'P':
		case 'p':
			portptr = &porttalbot;
			portname = "Port Talbot";
			break;

		case 'S':
		case 's':
			/* default port anyway */
			break;

		default:
			fprintf(stderr, "Unknown arg: %s\n", *ap);
			usage();
			return -1;
		}

		ap++;
		argc--;
	}

	if (argc == 1) {
		d = tptr->tm_mday;
		m = tptr->tm_mon+1;
		y = tptr->tm_year+1900;
	}
	else if (argc == 4) {
		tptr->tm_sec = 0;
		tptr->tm_min = 0;
		tptr->tm_hour = 12;
		tptr->tm_mday = atoi(ap[0]);
		tptr->tm_mon = atoi(ap[1]) - 1;
		tptr->tm_year = atoi(ap[2]) - 1900;
		tptr->tm_isdst = -1;
		
		if (atoi(ap[2]) < 1985)
			fprintf(stderr, "\aWarning: Tide tables probably not reliable before 1985\n");
			
		if ((now = mktime(tptr)) == -1) {
			fprintf(stderr, "Illegal or out-of-range date given.\n");
			usage();
			return -1;
		}
		
		tptr = localtime(&now);
		
		d = tptr->tm_mday;
		m = tptr->tm_mon+1;
		y = tptr->tm_year+1900;
	}
	else {
		usage();
		return -1;
	}

# ifdef MSDOS
	/* handle the TZ environment variable if possible */
	if ((tzptr = strncpy(tzbuffer, getenv("TZ"), sizeof tzbuffer)) && strlen(tzbuffer) >= 4) {
		tz = atoi(tzptr+3);

		/* Isolate correct timezone mnemonic */
		if (tptr->tm_isdst) {
			/* find DST mnemonic - assume DST is -1 hour */
			tzptr += 3;
			if (*tzptr == '+' || *tzptr == '-')
				tzptr++;

			while (isdigit(*tzptr))
				tzptr++;

			tz--;
		}

		if (*tzptr == '\0') {
			/* No DST mnemonic given? Use normal one minus 1 */
			tzptr = tzbuffer;
			strcpy(tzbuffer+3, "-01:00");
		}
		else if (strlen(tzptr) > 3)
			/* null-terminate mnemonic */
			tzptr[3] = '\0';
	}
	else {
		/* must assume GMT */
		tzptr = "GMT";
		tz = 0;
	}
# else
	/* set tzptr and tz to reflect summer/winter time of the date in question */
	tzptr = tzname[(tptr->tm_isdst > 0)? 1: 0];
	tz = winterzone - ((tptr->tm_isdst > 0)? 1: 0);
# endif

	printf("Tides at %s - %3s %02d %3s %04d\n", portname, wdaynames[tptr->tm_wday], d, mnames[m-1], y);
	printf("Times in %s.\n\n", tzptr);
	tmp = mkfactors(12.0 + tz, d, m, y);
	printf("Moon phase: ");
	pr_mphase(tmp->mphase, 10);

	tides = search4(tz, d, m, y, portptr);
	pr_tide(tides.t[0], tz);
	pr_tide(tides.t[1], tz);
	pr_tide(tides.t[2], tz);

	if (tides.t[3].height != 0.0)
		pr_tide(tides.t[3], tz);

	return 0;
}

static void
credits( void )
{

	printf(
	"Gower Tide Tables Program V4.0, Jul 1999.\n"
	"Written by Steven Hosgood: Copyright (c) S.Hosgood, 1988,1994,1996,1999.\n"
	"Data digitisation by Andrew Jenkins.\n"
	"\nBased upon:\n"
	"\tBritish Admiralty Simplified Harmonic Method, (document NP159)\n"
	"\nReferences:\n"
	"\tDoodson A.T & Warburg H.D: 'Admiralty Manual of Tides', [GC301.Adm]\n"
	"\tGodin G: 'The Analysis of Tides', [GC301.God]\n"
		);
	printf(
	"\nVersion History:\n"
	"1.0:\tOriginal, June 1988\n"
	"2.0:\t'M1' component included in combined 'O1' tide factor,\n"
	"\tAdded moon phase print-out,\n"
	"\tNew Swansea constants from Admiralty Tide Tables,\n"
	"\tAdded Mumbles, Burry Port, Port Talbot options, June 1994\n"
	"3.0:\tLinux port, May 1996\n"
	"4.0:\tMinor millennium bug fixed (just in time!),\n"
	"\tLinux/glibc DST-handling bugs fixed,\n"
	"\tAdded experimental Caswell Bay coefficients, July 1999\n"
		);
	printf(
	"\nThis version of this program may freely be copied and distributed\n"
	"as long as the credits and copyright message are kept intact, and\n"
	"that no charge is made for the copies. No guarantees as to the\n"
	"accuracy of the program are made, and none are to be implied.\n"
	"The program is for entertainment purposes only.\n"
	"\n"
		);

	return;
}


static void
usage ( void )
{

	printf("Usage: tides [args] <dd> <mm> <yyyy>\n");
	printf("   or: tides [args]\n");
	printf("\nArgs:\n");
	printf("\t-? or -h\tHelp Screen\n");
	printf("\t-b\t\tBurry Port\n");
	printf("\t-c\t\tCaswell Bay (warning: highly experimental!)\n");
	printf("\t-m\t\tMumbles Slip\n");
	printf("\t-p\t\tPort Talbot\n");
	printf("\t-s\t\tSwansea Docks (default)\n");
	return;
}
