/*
 * Defines file for structured data in tidal calculations.
 *
 * S.Hosgood 9.jly.88
 * Version 2.0:	Moon phase added to Factors struct.	21.May.94
 */

typedef struct {
	double		t_sine;
	double		t_cosine;
} Vector;

typedef struct {
	double		mphase;
	Vector		m2;
	Vector		s2;
	Vector		k1;
	Vector		o1;
} Factors;

typedef struct {
	double	z0;
	int	m2g;
	double	m2H;
	int	s2g;
	double	s2H;
	int	k1g;
	double	k1H;
	int	o1g;
	double	o1H;
	int	f4;
	double	F4;
	int	f6;
	double	F6;
} Port;

typedef struct {
	double	height;
	double	slope;
	double	time;
} Tide;

typedef struct {
	Tide	t[4];
} Tideset;

Factors	*mkfactors(double time, int d, int m, int y);
Tide	mktide(double time, int d, int m, int y, Port *portptr);
Tide	search( Tide a, Tide b, int d, int m, int y, Port *portptr );
Tideset	search4( int tz, int d, int m, int y, Port *portptr );
void	pr_mphase( double angle, int tolerance );
void	pr_tide( Tide t, int tz );
void	fhddr( void );

typedef struct {
	Port *port;
	char *name;
	double lati;
	double longi;
} Port_info;
